// Copyright 2000-2018 JetBrains s.r.o. Use of this source code is governed by the Apache 2.0 license that can be found in the LICENSE file.
package org.jetbrains.idea.devkit.completion;

import com.intellij.util.containers.ContainerUtil;
import org.jetbrains.annotations.ApiStatus;
import org.jetbrains.idea.devkit.themes.ThemeJsonCompletionContributor;

import java.awt.*;
import java.util.Set;

/**
 * @see org.jetbrains.idea.devkit.inspections.UnregisteredNamedColorInspection
 * @see ThemeJsonCompletionContributor
 * @see javax.swing.UIDefaults
 * @see com.intellij.ui.JBColor#namedColor(String, Color)
 * @see com.intellij.ui.JBColor#namedColor(String, int)
 */
@ApiStatus.Experimental
public final class UiDefaultsHardcodedKeys {
  // hardcoded because these keys can differ from platform to platform and from theme to theme.
  @SuppressWarnings("SpellCheckingInspection")
  public static final Set<String> UI_DEFAULTS_KEYS = ContainerUtil.immutableSet(
    "AATextInfoPropertyKey",
    "AbstractButton.click.textAndMnemonic",
    "AbstractDocument.addition.textAndMnemonic",
    "AbstractDocument.deletion.textAndMnemonic",
    "AbstractDocument.redo.textAndMnemonic",
    "AbstractDocument.styleChange.textAndMnemonic",
    "AbstractDocument.undo.textAndMnemonic",
    "AbstractUndoableEdit.redo.textAndMnemonic",
    "AbstractUndoableEdit.undo.textAndMnemonic",
    "ActionButton.hoverBackground",
    "ActionButton.hoverBorderColor",
    "ActionButton.pressedBackground",
    "ActionButton.pressedBorderColor",
    "AuditoryCues.allAuditoryCues",
    "AuditoryCues.cueList",
    "AuditoryCues.defaultCueList",
    "AuditoryCues.noAuditoryCues",
    "Balloon.error.textInsets",
    "Component.focusWidth",
    "Button.actionMap",
    "Button.arc",
    "Button.background",
    "Button.background.opaque",
    "Button.border",
    "Button.default.shadowColor",
    "Button.darcula.outlineColor",
    "Button.paintShadow",
    "Button.shadowWidth",
    "Button.darkShadow",
    "Button.default.borderColor",
    "Button.default.endBackground",
    "Button.default.endBorderColor",
    "Button.default.focusColor",
    "Button.default.focusedBorderColor",
    "Button.default.foreground",
    "Button.default.startBackground",
    "Button.default.startBorderColor",
    "Button.defaultButtonFollowsFocus",
    "Button.disabledBorderColor",
    "Button.disabledText",
    "Button.disabledToolBarBorderBackground",
    "Button.endBackground",
    "Button.endBorderColor",
    "Button.focus",
    "Button.focusInputMap",
    "Button.focusedBorderColor",
    "Button.font",
    "Button.foreground",
    "Button.gradient",
    "Button.highlight",
    "Button.intellij.native.borderColor",
    "Button.intellij.native.focusDottedRingColor",
    "Button.intellij.native.focusedBackgroundColor",
    "Button.intellij.native.focusedBorderColor",
    "Button.intellij.native.pressedBackgroundColor",
    "Button.intellij.native.pressedBorderColor",
    "Button.light",
    "Button.margin",
    "Button.opaque",
    "Button.rollover",
    "Button.rolloverIconType",
    "Button.select",
    "Button.shadow",
    "Button.shadowColor",
    "Button.startBackground",
    "Button.startBorderColor",
    "Button.textIconGap",
    "Button.textShiftOffset",
    "Button.toolBarBorderBackground",
    "ButtonUI",
    "CellEditor.border.width",
    "CheckBox.background",
    "CheckBox.border",
    "CheckBox.borderInsets",
    "CheckBox.disabledText",
    "CheckBox.focus",
    "CheckBox.focusInputMap",
    "CheckBox.font",
    "CheckBox.foreground",
    "CheckBox.gradient",
    "CheckBox.icon",
    "CheckBox.margin",
    "CheckBox.rollover",
    "CheckBox.select",
    "CheckBox.textIconGap",
    "CheckBox.textShiftOffset",
    "CheckBox.totalInsets",
    "CheckBoxMenuItem.acceleratorDelimiter",
    "CheckBoxMenuItem.acceleratorFont",
    "CheckBoxMenuItem.acceleratorForeground",
    "CheckBoxMenuItem.acceleratorSelectionForeground",
    "CheckBoxMenuItem.arrowIcon",
    "CheckBoxMenuItem.background",
    "CheckBoxMenuItem.border",
    "CheckBoxMenuItem.borderPainted",
    "CheckBoxMenuItem.checkIcon",
    "CheckBoxMenuItem.commandSound",
    "CheckBoxMenuItem.dashIcon",
    "CheckBoxMenuItem.disabledBackground",
    "CheckBoxMenuItem.disabledForeground",
    "CheckBoxMenuItem.font",
    "CheckBoxMenuItem.foreground",
    "CheckBoxMenuItem.gradient",
    "CheckBoxMenuItem.margin",
    "CheckBoxMenuItem.selectionBackground",
    "CheckBoxMenuItem.selectionForeground",
    "CheckBoxMenuItemUI",
    "CheckBoxUI",
    "Checkbox.select",
    "ColorChooser.background",
    "ColorChooser.cancel.textAndMnemonic",
    "ColorChooser.font",
    "ColorChooser.foreground",
    "ColorChooser.hsb.textAndMnemonic",
    "ColorChooser.hsbBlue.textAndMnemonic",
    "ColorChooser.hsbBrightness.textAndMnemonic",
    "ColorChooser.hsbGreen.textAndMnemonic",
    "ColorChooser.hsbHue.textAndMnemonic",
    "ColorChooser.hsbRed.textAndMnemonic",
    "ColorChooser.hsbSaturation.textAndMnemonic",
    "ColorChooser.ok.textAndMnemonic",
    "ColorChooser.preview.textAndMnemonic",
    "ColorChooser.reset.textAndMnemonic",
    "ColorChooser.rgb.textAndMnemonic",
    "ColorChooser.rgbBlue.textAndMnemonic",
    "ColorChooser.rgbGreen.textAndMnemonic",
    "ColorChooser.rgbRed.textAndMnemonic",
    "ColorChooser.sample.textAndMnemonic",
    "ColorChooser.swatches.textAndMnemonic",
    "ColorChooser.swatchesDefaultRecentColor",
    "ColorChooser.swatchesRecent.textAndMnemonic",
    "ColorChooser.swatchesRecentSwatchSize",
    "ColorChooser.swatchesSwatchSize",
    "ColorChooserUI",
    "ComboBox.ArrowButton.background",
    "ComboBox.ArrowButton.disabledIconColor",
    "ComboBox.ArrowButton.iconColor",
    "ComboBox.ArrowButton.nonEditableBackground",
    "ComboBox.actionMap",
    "ComboBox.ancestorInputMap",
    "ComboBox.background",
    "ComboBox.border",
    "ComboBox.buttonBackground",
    "ComboBox.buttonDarkShadow",
    "ComboBox.buttonHighlight",
    "ComboBox.buttonShadow",
    "ComboBox.disabledBackground",
    "ComboBox.disabledForeground",
    "ComboBox.font",
    "ComboBox.foreground",
    "ComboBox.isEnterSelectablePopup",
    "ComboBox.noActionOnKeyNavigation",
    "ComboBox.nonEditableBackground",
    "ComboBox.padding",
    "ComboBox.selectionBackground",
    "ComboBox.selectionForeground",
    "ComboBox.timeFactor",
    "ComboBox.togglePopup.textAndMnemonic",
    "ComboBoxButton.background",
    "ComboBoxUI",
    "Component.arc",
    "Component.borderColor",
    "Component.disabledBorderColor",
    "Component.errorFocusColor",
    "Component.focusColor",
    "Component.focusedBorderColor",
    "Component.hoverIconColor",
    "Component.iconColor",
    "Component.inactiveErrorFocusColor",
    "Component.inactiveWarningFocusColor",
    "Component.infoForeground",
    "Component.warningFocusColor",
    "Desktop.ancestorInputMap",
    "Desktop.background",
    "Desktop.minOnScreenInsets",
    "DesktopIcon.background",
    "DesktopIcon.border",
    "DesktopIcon.borderColor",
    "DesktopIcon.borderRimColor",
    "DesktopIcon.font",
    "DesktopIcon.foreground",
    "DesktopIcon.labelBackground",
    "DesktopIcon.width",
    "DesktopIconUI",
    "DesktopPaneUI",
    "DialogWrapper.southPanelBackground",
    "DialogWrapper.southPanelDivider",
    "DragAndDrop.areaBackground",
    "DragAndDrop.areaBorderColor",
    "DragAndDrop.areaForeground",
    "EditorPane.background",
    "EditorPane.border",
    "EditorPane.caretBlinkRate",
    "EditorPane.caretForeground",
    "EditorPane.focusInputMap",
    "EditorPane.font",
    "EditorPane.foreground",
    "EditorPane.inactiveBackground",
    "EditorPane.inactiveForeground",
    "EditorPane.margin",
    "EditorPane.selectionBackground",
    "EditorPane.selectionForeground",
    "EditorPaneUI",
    "FileChooser.acceptAllFileFilter.textAndMnemonic",
    "FileChooser.ancestorInputMap",
    "FileChooser.by.textAndMnemonic",
    "FileChooser.byDate.textAndMnemonic",
    "FileChooser.cancelButton.textAndMnemonic",
    "FileChooser.cancelButtonMnemonic",
    "FileChooser.chooseButton.textAndMnemonic",
    "FileChooser.createButton.textAndMnemonic",
    "FileChooser.desktopName",
    "FileChooser.detailsViewActionLabelText",
    "FileChooser.detailsViewIcon",
    "FileChooser.directoryDescription.textAndMnemonic",
    "FileChooser.directoryOpenButton.textAndMnemonic",
    "FileChooser.directoryOpenButtonMnemonic",
    "FileChooser.fileDescription.textAndMnemonic",
    "FileChooser.fileNameLabel.textAndMnemonic",
    "FileChooser.fileNameLabelMnemonic",
    "FileChooser.fileSizeGigaBytes",
    "FileChooser.fileSizeKiloBytes",
    "FileChooser.fileSizeMegaBytes",
    "FileChooser.filesOfTypeLabel.textAndMnemonic",
    "FileChooser.filesOfTypeLabelMnemonic",
    "FileChooser.helpButton.textAndMnemonic",
    "FileChooser.helpButtonMnemonic",
    "FileChooser.homeFolderIcon",
    "FileChooser.listViewActionLabelText",
    "FileChooser.listViewIcon",
    "FileChooser.lookInLabelMnemonic",
    "FileChooser.mac.newFolder",
    "FileChooser.mac.newFolder.subsequent",
    "FileChooser.newFolderAccessibleName",
    "FileChooser.newFolderActionLabelText",
    "FileChooser.newFolderButton.textAndMnemonic",
    "FileChooser.newFolderError.textAndMnemonic",
    "FileChooser.newFolderErrorSeparator",
    "FileChooser.newFolderExistsError.textAndMnemonic",
    "FileChooser.newFolderIcon",
    "FileChooser.newFolderPrompt.textAndMnemonic",
    "FileChooser.newFolderTitle.textAndMnemonic",
    "FileChooser.openButton.textAndMnemonic",
    "FileChooser.openButtonMnemonic",
    "FileChooser.openDialogTitle.textAndMnemonic",
    "FileChooser.openTitle.textAndMnemonic",
    "FileChooser.readOnly",
    "FileChooser.refreshActionLabelText",
    "FileChooser.saveButton.textAndMnemonic",
    "FileChooser.saveButtonMnemonic",
    "FileChooser.saveDialogFileNameLabel.textAndMnemonic",
    "FileChooser.saveDialogTitle.textAndMnemonic",
    "FileChooser.saveTitle.textAndMnemonic",
    "FileChooser.untitledFileName",
    "FileChooser.untitledFolderName",
    "FileChooser.upFolderIcon",
    "FileChooser.updateButton.textAndMnemonic",
    "FileChooser.updateButtonMnemonic",
    "FileChooser.useSystemExtensionHiding",
    "FileChooser.usesSingleFilePane",
    "FileChooser.viewMenuLabelText",
    "FileChooserUI",
    "FileView.computerIcon",
    "FileView.directoryIcon",
    "FileView.fileIcon",
    "FileView.floppyDriveIcon",
    "FileView.hardDriveIcon",
    "Focus.color",
    "FormView.browseFileButton.textAndMnemonic",
    "FormView.resetButton.textAndMnemonic",
    "FormView.submitButton.textAndMnemonic",
    "FormattedTextField.actionMap",
    "FormattedTextField.background",
    "FormattedTextField.border",
    "FormattedTextField.caretBlinkRate",
    "FormattedTextField.caretForeground",
    "FormattedTextField.focusInputMap",
    "FormattedTextField.font",
    "FormattedTextField.foreground",
    "FormattedTextField.inactiveBackground",
    "FormattedTextField.inactiveForeground",
    "FormattedTextField.margin",
    "FormattedTextField.selectionBackground",
    "FormattedTextField.selectionForeground",
    "FormattedTextFieldUI",
    "Group.separatorColor",
    "HelpTooltip.background",
    "HelpTooltip.borderColor",
    "HelpTooltip.defaultTextBorderInsets",
    "HelpTooltip.fontSizeDelta",
    "HelpTooltip.foreground",
    "HelpTooltip.horizontalGap",
    "HelpTooltip.maxWidth",
    "HelpTooltip.shortcutForeground",
    "HelpTooltip.smallTextBorderInsets",
    "HelpTooltip.verticalGap",
    "HelpTooltip.xOffset",
    "HelpTooltip.yOffset",
    "Hyperlink.linkColor",
    "IconButton.font",
    "InsetBorder.aquaVariant",
    "InternalFrame.activeTitleBackground",
    "InternalFrame.activeTitleForeground",
    "InternalFrame.activeTitleGradient",
    "InternalFrame.background",
    "InternalFrame.border",
    "InternalFrame.borderColor",
    "InternalFrame.borderDarkShadow",
    "InternalFrame.borderHighlight",
    "InternalFrame.borderLight",
    "InternalFrame.borderShadow",
    "InternalFrame.closeButtonToolTip",
    "InternalFrame.closeIcon",
    "InternalFrame.closeSound",
    "InternalFrame.icon",
    "InternalFrame.iconButtonToolTip",
    "InternalFrame.iconifyIcon",
    "InternalFrame.inactiveTitleBackground",
    "InternalFrame.inactiveTitleForeground",
    "InternalFrame.maxButtonToolTip",
    "InternalFrame.maximizeIcon",
    "InternalFrame.maximizeSound",
    "InternalFrame.minimizeIcon",
    "InternalFrame.minimizeSound",
    "InternalFrame.opaque",
    "InternalFrame.optionDialogBackground",
    "InternalFrame.optionDialogBorder",
    "InternalFrame.optionDialogTitleFont",
    "InternalFrame.paletteBackground",
    "InternalFrame.paletteBorder",
    "InternalFrame.paletteCloseIcon",
    "InternalFrame.paletteTitleFont",
    "InternalFrame.paletteTitleHeight",
    "InternalFrame.restoreButtonToolTip",
    "InternalFrame.restoreDownSound",
    "InternalFrame.restoreUpSound",
    "InternalFrame.titleFont",
    "InternalFrame.windowBindings",
    "InternalFrameTitlePane.closeButton.textAndMnemonic",
    "InternalFrameTitlePane.closeButtonAccessibleName",
    "InternalFrameTitlePane.closeButtonOpacity",
    "InternalFrameTitlePane.iconifyButtonAccessibleName",
    "InternalFrameTitlePane.iconifyButtonOpacity",
    "InternalFrameTitlePane.maximizeButton.textAndMnemonic",
    "InternalFrameTitlePane.maximizeButtonAccessibleName",
    "InternalFrameTitlePane.maximizeButtonOpacity",
    "InternalFrameTitlePane.minimizeButton.textAndMnemonic",
    "InternalFrameTitlePane.moveButton.textAndMnemonic",
    "InternalFrameTitlePane.restoreButton.textAndMnemonic",
    "InternalFrameTitlePane.sizeButton.textAndMnemonic",
    "InternalFrameUI",
    "IsindexView.prompt",
    "Label.actionMap",
    "Label.background",
    "Label.disabledForeground",
    "Label.disabledShadow",
    "Label.font",
    "Label.foreground",
    "Label.opaque",
    "Label.selectedDisabledForeground",
    "Label.selectedForeground",
    "LabelUI",
    "Link.activeForeground",
    "Link.hoverForeground",
    "Link.pressedForeground",
    "Link.visitedForeground",
    "List.actionMap",
    "List.background",
    "List.cellRenderer",
    "List.dropCellBackground",
    "List.dropLineColor",
    "List.evenRowBackgroundPainter",
    "List.focusCellHighlightBorder",
    "List.focusInputMap",
    "List.focusInputMap.RightToLeft",
    "List.font",
    "List.foreground",
    "List.noFocusBorder",
    "List.oddRowBackgroundPainter",
    "List.selectionBackground",
    "List.selectionForeground",
    "List.selectionInactiveBackground",
    "List.selectionInactiveForeground",
    "List.sourceListBackgroundPainter",
    "List.sourceListFocusedSelectionBackgroundPainter",
    "List.sourceListSelectionBackgroundPainter",
    "List.timeFactor",
    "ListUI",
    "Menu.acceleratorFont",
    "Menu.acceleratorForeground",
    "Menu.acceleratorSelectionForeground",
    "Menu.actionMap",
    "Menu.arrowIcon",
    "Menu.background",
    "Menu.border",
    "Menu.borderColor",
    "Menu.borderPainted",
    "Menu.cancelMode",
    "Menu.checkIcon",
    "Menu.consumesTabs",
    "Menu.crossMenuMnemonic",
    "Menu.disabledArrowIcon",
    "Menu.disabledBackground",
    "Menu.disabledForeground",
    "Menu.font",
    "Menu.foreground",
    "Menu.invertedArrowIcon",
    "Menu.margin",
    "Menu.maxGutterIconWidth",
    "Menu.menuPopupOffsetX",
    "Menu.menuPopupOffsetY",
    "Menu.opaque",
    "Menu.preserveTopLevelSelection",
    "Menu.selectionBackground",
    "Menu.selectionForeground",
    "Menu.separatorColor",
    "Menu.shortcutKeys",
    "Menu.submenuPopupOffsetX",
    "Menu.submenuPopupOffsetY",
    "MenuBar.actionMap",
    "MenuBar.background",
    "MenuBar.backgroundPainter",
    "MenuBar.border",
    "MenuBar.borderColor",
    "MenuBar.darcula.borderColor",
    "MenuBar.darcula.borderShadowColor",
    "MenuBar.disabledBackground",
    "MenuBar.disabledForeground",
    "MenuBar.font",
    "MenuBar.foreground",
    "MenuBar.gradient",
    "MenuBar.highlight",
    "MenuBar.margin",
    "MenuBar.selectedBackgroundPainter",
    "MenuBar.selectionBackground",
    "MenuBar.selectionForeground",
    "MenuBar.shadow",
    "MenuBar.windowBindings",
    "MenuBarUI",
    "MenuItem.acceleratorDelimiter",
    "MenuItem.acceleratorFont",
    "MenuItem.acceleratorForeground",
    "MenuItem.acceleratorSelectionForeground",
    "MenuItem.actionMap",
    "MenuItem.arrowIcon",
    "MenuItem.background",
    "MenuItem.border",
    "MenuItem.borderPainted",
    "MenuItem.checkIcon",
    "MenuItem.commandSound",
    "MenuItem.disabledBackground",
    "MenuItem.disabledForeground",
    "MenuItem.font",
    "MenuItem.foreground",
    "MenuItem.margin",
    "MenuItem.maxGutterIconWidth",
    "MenuItem.selectedBackgroundPainter",
    "MenuItem.selectionBackground",
    "MenuItem.selectionForeground",
    "MenuItemUI",
    "MenuUI",
    "NavBar.borderColor",
    "OnOffButtonUI",
    "OnePixelDivider.background",
    "OptionButtonUI",
    "OptionPane.background",
    "OptionPane.border",
    "OptionPane.buttonAreaBorder",
    "OptionPane.buttonClickThreshhold",
    "OptionPane.buttonFont",
    "OptionPane.cancelButton.textAndMnemonic",
    "OptionPane.cancelButtonMnemonic",
    "OptionPane.errorDialog.border.background",
    "OptionPane.errorDialog.titlePane.background",
    "OptionPane.errorDialog.titlePane.foreground",
    "OptionPane.errorDialog.titlePane.shadow",
    "OptionPane.errorIcon",
    "OptionPane.errorSound",
    "OptionPane.font",
    "OptionPane.foreground",
    "OptionPane.informationIcon",
    "OptionPane.informationSound",
    "OptionPane.inputDialog.titleAndMnemonic",
    "OptionPane.messageAreaBorder",
    "OptionPane.messageDialog.titleAndMnemonic",
    "OptionPane.messageFont",
    "OptionPane.messageForeground",
    "OptionPane.minimumSize",
    "OptionPane.noButton.textAndMnemonic",
    "OptionPane.noButtonMnemonic",
    "OptionPane.okButton.textAndMnemonic",
    "OptionPane.okButtonMnemonic",
    "OptionPane.questionDialog.border.background",
    "OptionPane.questionDialog.titlePane.background",
    "OptionPane.questionDialog.titlePane.foreground",
    "OptionPane.questionDialog.titlePane.shadow",
    "OptionPane.questionIcon",
    "OptionPane.questionSound",
    "OptionPane.title.textAndMnemonic",
    "OptionPane.warningDialog.border.background",
    "OptionPane.warningDialog.titlePane.background",
    "OptionPane.warningDialog.titlePane.foreground",
    "OptionPane.warningDialog.titlePane.shadow",
    "OptionPane.warningIcon",
    "OptionPane.warningSound",
    "OptionPane.windowBindings",
    "OptionPane.yesButton.textAndMnemonic",
    "OptionPane.yesButtonMnemonic",
    "OptionPaneUI",
    "Panel.background",
    "Panel.font",
    "Panel.foreground",
    "Panel.opaque",
    "PanelUI",
    "PasswordField.actionMap",
    "PasswordField.background",
    "PasswordField.border",
    "PasswordField.capsLockIconColor",
    "PasswordField.caretBlinkRate",
    "PasswordField.caretForeground",
    "PasswordField.echoChar",
    "PasswordField.focusInputMap",
    "PasswordField.font",
    "PasswordField.foreground",
    "PasswordField.inactiveBackground",
    "PasswordField.inactiveForeground",
    "PasswordField.margin",
    "PasswordField.selectionBackground",
    "PasswordField.selectionForeground",
    "PasswordFieldUI",
    "Popup.Header.activeBackground",
    "Popup.Header.inactiveBackground",
    "Popup.Toolbar.background",
    "Popup.Toolbar.borderColor",
    "Popup.borderColor",
    "Popup.inactiveBorderColor",
    "PopupMenu.actionMap",
    "PopupMenu.background",
    "PopupMenu.border",
    "PopupMenu.borderInsets",
    "PopupMenu.borderWidth",
    "PopupMenu.consumeEventOnClose",
    "PopupMenu.font",
    "PopupMenu.foreground",
    "PopupMenu.popupSound",
    "PopupMenu.selectedWindowInputMapBindings",
    "PopupMenu.selectedWindowInputMapBindings.RightToLeft",
    "PopupMenu.selectionBackground",
    "PopupMenu.selectionForeground",
    "PopupMenu.translucentBackground",
    "PopupMenuSeparator.height",
    "PopupMenuSeparator.stripeIndent",
    "PopupMenuSeparator.stripeWidth",
    "PopupMenuSeparatorUI",
    "PopupMenuUI",
    "PrintingDialog.abortButton.textAndMnemonic",
    "PrintingDialog.abortButtonToolTip.textAndMnemonic",
    "PrintingDialog.contentAborting.textAndMnemonic",
    "PrintingDialog.contentInitial.textAndMnemonic",
    "PrintingDialog.contentProgress.textAndMnemonic",
    "PrintingDialog.titleAborting.textAndMnemonic",
    "PrintingDialog.titleProgress.textAndMnemonic",
    "ProgressBar.background",
    "ProgressBar.border",
    "ProgressBar.cellLength",
    "ProgressBar.cellSpacing",
    "ProgressBar.cycleTime",
    "ProgressBar.failedColor",
    "ProgressBar.failedEndColor",
    "ProgressBar.font",
    "ProgressBar.foreground",
    "ProgressBar.horizontalSize",
    "ProgressBar.indeterminateEndColor",
    "ProgressBar.indeterminateStartColor",
    "ProgressBar.passedColor",
    "ProgressBar.passedEndColor",
    "ProgressBar.progressColor",
    "ProgressBar.repaintInterval",
    "ProgressBar.selectionBackground",
    "ProgressBar.selectionForeground",
    "ProgressBar.trackColor",
    "ProgressBar.verticalSize",
    "ProgressBarUI",
    "ProgressMonitor.progress.textAndMnemonic",
    "RadioButton.background",
    "RadioButton.border",
    "RadioButton.borderInsets",
    "RadioButton.darkShadow",
    "RadioButton.disabledText",
    "RadioButton.focus",
    "RadioButton.focusInputMap",
    "RadioButton.font",
    "RadioButton.foreground",
    "RadioButton.gradient",
    "RadioButton.highlight",
    "RadioButton.icon",
    "RadioButton.light",
    "RadioButton.margin",
    "RadioButton.rollover",
    "RadioButton.select",
    "RadioButton.shadow",
    "RadioButton.textIconGap",
    "RadioButton.textShiftOffset",
    "RadioButton.totalInsets",
    "RadioButtonMenuItem.acceleratorDelimiter",
    "RadioButtonMenuItem.acceleratorFont",
    "RadioButtonMenuItem.acceleratorForeground",
    "RadioButtonMenuItem.acceleratorSelectionForeground",
    "RadioButtonMenuItem.arrowIcon",
    "RadioButtonMenuItem.background",
    "RadioButtonMenuItem.border",
    "RadioButtonMenuItem.borderPainted",
    "RadioButtonMenuItem.checkIcon",
    "RadioButtonMenuItem.commandSound",
    "RadioButtonMenuItem.dashIcon",
    "RadioButtonMenuItem.disabledBackground",
    "RadioButtonMenuItem.disabledForeground",
    "RadioButtonMenuItem.font",
    "RadioButtonMenuItem.foreground",
    "RadioButtonMenuItem.gradient",
    "RadioButtonMenuItem.margin",
    "RadioButtonMenuItem.selectionBackground",
    "RadioButtonMenuItem.selectionForeground",
    "RadioButtonMenuItemUI",
    "RadioButtonUI",
    "RootPane.actionMap",
    "RootPane.ancestorInputMap",
    "RootPane.colorChooserDialogBorder",
    "RootPane.defaultButtonWindowKeyBindings",
    "RootPane.errorDialogBorder",
    "RootPane.fileChooserDialogBorder",
    "RootPane.frameBorder",
    "RootPane.informationDialogBorder",
    "RootPane.plainDialogBorder",
    "RootPane.questionDialogBorder",
    "RootPane.warningDialogBorder",
    "RootPaneUI",
    "ScrollBar.actionMap",
    "ScrollBar.allowsAbsolutePositioning",
    "ScrollBar.ancestorInputMap",
    "ScrollBar.ancestorInputMap.RightToLeft",
    "ScrollBar.background",
    "ScrollBar.trackColor",
    "ScrollBar.thumbColor",
    "ScrollBar.thumbBorderColor",
    "ScrollBar.hoverTrackColor",
    "ScrollBar.hoverThumbColor",
    "ScrollBar.hoverThumbBorderColor",
    "ScrollBar.Transparent.trackColor",
    "ScrollBar.Transparent.thumbColor",
    "ScrollBar.Transparent.thumbBorderColor",
    "ScrollBar.Transparent.hoverTrackColor",
    "ScrollBar.Transparent.hoverThumbColor",
    "ScrollBar.Transparent.hoverThumbBorderColor",
    "ScrollBar.Mac.trackColor",
    "ScrollBar.Mac.thumbColor",
    "ScrollBar.Mac.thumbBorderColor",
    "ScrollBar.Mac.hoverTrackColor",
    "ScrollBar.Mac.hoverThumbColor",
    "ScrollBar.Mac.hoverThumbBorderColor",
    "ScrollBar.Mac.Transparent.trackColor",
    "ScrollBar.Mac.Transparent.thumbColor",
    "ScrollBar.Mac.Transparent.thumbBorderColor",
    "ScrollBar.Mac.Transparent.hoverTrackColor",
    "ScrollBar.Mac.Transparent.hoverThumbColor",
    "ScrollBar.Mac.Transparent.hoverThumbBorderColor",
    "ScrollBar.darkShadow",
    "ScrollBar.decrementButtonGap",
    "ScrollBar.focusInputMap",
    "ScrollBar.focusInputMap.RightToLeft",
    "ScrollBar.foreground",
    "ScrollBar.gradient",
    "ScrollBar.highlight",
    "ScrollBar.incrementButtonGap",
    "ScrollBar.maximumThumbSize",
    "ScrollBar.minimumThumbSize",
    "ScrollBar.shadow",
    "ScrollBar.thumb",
    "ScrollBar.thumbDarkShadow",
    "ScrollBar.thumbHighlight",
    "ScrollBar.thumbShadow",
    "ScrollBar.track",
    "ScrollBar.trackHighlight",
    "ScrollBar.width",
    "ScrollBarUI",
    "ScrollPane.actionMap",
    "ScrollPane.ancestorInputMap",
    "ScrollPane.ancestorInputMap.RightToLeft",
    "ScrollPane.background",
    "ScrollPane.border",
    "ScrollPane.font",
    "ScrollPane.foreground",
    "ScrollPaneUI",
    "SearchEverywhere.Advertiser.background",
    "SearchEverywhere.Advertiser.foreground",
    "SearchEverywhere.Header.background",
    "SearchEverywhere.List.separatorColor",
    "SearchEverywhere.SearchField.background",
    "SearchEverywhere.SearchField.borderColor",
    "SearchEverywhere.SearchField.infoForeground",
    "SearchEverywhere.Tab.selectedBackground",
    "SearchEverywhere.Tab.selectedForeground",
    "SearchEverywhere.background",
    "SearchEverywhere.foreground",
    "SearchEverywhere.shortcutForeground",
    "Separator.background",
    "Separator.foreground",
    "Separator.highlight",
    "Separator.separatorColor",
    "Separator.shadow",
    "SeparatorUI",
    "Slider.actionMap",
    "Slider.altTrackColor",
    "Slider.background",
    "Slider.disabledForeground",
    "Slider.focus",
    "Slider.focusGradient",
    "Slider.focusInputMap",
    "Slider.focusInputMap.RightToLeft",
    "Slider.focusInsets",
    "Slider.focusedColor",
    "Slider.font",
    "Slider.foreground",
    "Slider.gradient",
    "Slider.highlight",
    "Slider.horizontalSize",
    "Slider.horizontalThumbIcon",
    "Slider.majorTickLength",
    "Slider.minimumHorizontalSize",
    "Slider.minimumVerticalSize",
    "Slider.onlyLeftMouseButtonDrag",
    "Slider.pressedColor",
    "Slider.shadow",
    "Slider.tickColor",
    "Slider.trackWidth",
    "Slider.verticalSize",
    "Slider.verticalThumbIcon",
    "SliderUI",
    "Spinner.actionMap",
    "Spinner.ancestorInputMap",
    "Spinner.arrowButtonBorder",
    "Spinner.arrowButtonInsets",
    "Spinner.arrowButtonSize",
    "Spinner.background",
    "Spinner.border",
    "Spinner.editorAlignment",
    "Spinner.editorBorderPainted",
    "Spinner.font",
    "Spinner.foreground",
    "SpinnerUI",
    "SplitPane.actionMap",
    "SplitPane.ancestorInputMap",
    "SplitPane.background",
    "SplitPane.border",
    "SplitPane.centerOneTouchButtons",
    "SplitPane.darkShadow",
    "SplitPane.dividerFocusColor",
    "SplitPane.dividerSize",
    "SplitPane.highlight",
    "SplitPane.leftButton.textAndMnemonic",
    "SplitPane.oneTouchButtonsOpaque",
    "SplitPane.rightButton.textAndMnemonic",
    "SplitPane.shadow",
    "SplitPaneDivider.border",
    "SplitPaneDivider.draggingColor",
    "SplitPaneDivider.horizontalGradientVariant",
    "SplitPaneUI",
    "StatusBar.borderColor",
    "StatusBar.bottomColor",
    "StatusBar.top2Color",
    "StatusBar.topColor",
    "StyledEditorKit.JBDefaultStyle",
    "TabbedPane.actionMap",
    "TabbedPane.ancestorInputMap",
    "TabbedPane.background",
    "TabbedPane.borderHightlightColor",
    "TabbedPane.contentAreaColor",
    "TabbedPane.contentBorderInsets",
    "TabbedPane.contentOpaque",
    "TabbedPane.darkShadow",
    "TabbedPane.disabledForeground",
    "TabbedPane.disabledUnderlineColor",
    "TabbedPane.focus",
    "TabbedPane.focusColor",
    "TabbedPane.focusInputMap",
    "TabbedPane.font",
    "TabbedPane.foreground",
    "TabbedPane.highlight",
    "TabbedPane.hoverColor",
    "TabbedPane.labelShift",
    "TabbedPane.leftTabInsets",
    "TabbedPane.light",
    "TabbedPane.nonSelectedTabTitleNormalColor",
    "TabbedPane.opaque",
    "TabbedPane.rightTabInsets",
    "TabbedPane.selectHighlight",
    "TabbedPane.selected",
    "TabbedPane.selectedLabelShift",
    "TabbedPane.selectedTabPadInsets",
    "TabbedPane.selectedTabTitleDisabledColor",
    "TabbedPane.selectedTabTitleNormalColor",
    "TabbedPane.selectedTabTitlePressedColor",
    "TabbedPane.selectedTabTitleShadowDisabledColor",
    "TabbedPane.selectedTabTitleShadowNormalColor",
    "TabbedPane.selectionFollowsFocus",
    "TabbedPane.shadow",
    "TabbedPane.smallFont",
    "TabbedPane.tabAreaBackground",
    "TabbedPane.tabAreaInsets",
    "TabbedPane.tabFillStyle",
    "TabbedPane.tabHeight",
    "TabbedPane.tabInsets",
    "TabbedPane.tabRunOverlay",
    "TabbedPane.tabSelectionHeight",
    "TabbedPane.tabsOpaque",
    "TabbedPane.tabsOverlapBorder",
    "TabbedPane.textIconGap",
    "TabbedPane.underlineColor",
    "TabbedPane.unselectedBackground",
    "TabbedPane.useSmallLayout",
    "TabbedPaneUI",
    "Table.actionMap",
    "Table.ancestorInputMap",
    "Table.ancestorInputMap.RightToLeft",
    "Table.ascendingSortIcon",
    "Table.background",
    "Table.cellNoFocusBorder",
    "Table.descendingSortIcon",
    "Table.dropCellBackground",
    "Table.dropLineColor",
    "Table.dropLineShortColor",
    "Table.focusCellBackground",
    "Table.focusCellForeground",
    "Table.focusCellHighlightBorder",
    "Table.focusSelectedCellHighlightBorder",
    "Table.font",
    "Table.foreground",
    "Table.gridColor",
    "Table.scrollPaneBorder",
    "Table.selectionBackground",
    "Table.selectionForeground",
    "Table.selectionInactiveBackground",
    "Table.selectionInactiveForeground",
    "Table.sortIconColor",
    "TableHeader.actionMap",
    "TableHeader.ancestorInputMap",
    "TableHeader.background",
    "TableHeader.cellBorder",
    "TableHeader.focusCellBackground",
    "TableHeader.font",
    "TableHeader.foreground",
    "TableHeaderUI",
    "TableUI",
    "TextArea.actionMap",
    "TextArea.background",
    "TextArea.border",
    "TextArea.caretBlinkRate",
    "TextArea.caretForeground",
    "TextArea.focusInputMap",
    "TextArea.font",
    "TextArea.foreground",
    "TextArea.inactiveBackground",
    "TextArea.inactiveForeground",
    "TextArea.margin",
    "TextArea.selectionBackground",
    "TextArea.selectionForeground",
    "TextAreaUI",
    "TextComponent.selectionBackgroundInactive",
    "TextField.actionMap",
    "TextField.background",
    "TextField.border",
    "TextField.borderColor",
    "TextField.caretBlinkRate",
    "TextField.caretForeground",
    "TextField.darkShadow",
    "TextField.disabledBackground",
    "TextField.focusInputMap",
    "TextField.focusedBorderColor",
    "TextField.font",
    "TextField.foreground",
    "TextField.highlight",
    "TextField.hoverBorderColor",
    "TextField.inactiveBackground",
    "TextField.inactiveForeground",
    "TextField.light",
    "TextField.margin",
    "TextField.selectionBackground",
    "TextField.selectionForeground",
    "TextField.shadow",
    "TextFieldUI",
    "TextPane.background",
    "TextPane.border",
    "TextPane.caretBlinkRate",
    "TextPane.caretForeground",
    "TextPane.focusInputMap",
    "TextPane.font",
    "TextPane.foreground",
    "TextPane.inactiveBackground",
    "TextPane.inactiveForeground",
    "TextPane.margin",
    "TextPane.selectionBackground",
    "TextPane.selectionForeground",
    "TextPaneUI",
    "TitledBorder.aquaVariant",
    "TitledBorder.border",
    "TitledBorder.font",
    "TitledBorder.titleColor",
    "ToggleButton.background",
    "ToggleButton.border",
    "ToggleButton.darkShadow",
    "ToggleButton.disabledText",
    "ToggleButton.focus",
    "ToggleButton.focusInputMap",
    "ToggleButton.font",
    "ToggleButton.foreground",
    "ToggleButton.gradient",
    "ToggleButton.highlight",
    "ToggleButton.light",
    "ToggleButton.margin",
    "ToggleButton.select",
    "ToggleButton.shadow",
    "ToggleButton.textIconGap",
    "ToggleButton.textShiftOffset",
    "ToggleButtonUI",
    "ToolBar.ancestorInputMap",
    "ToolBar.background",
    "ToolBar.border",
    "ToolBar.borderColor",
    "ToolBar.borderHandleColor",
    "ToolBar.darkShadow",
    "ToolBar.dockingBackground",
    "ToolBar.dockingForeground",
    "ToolBar.floatingBackground",
    "ToolBar.floatingForeground",
    "ToolBar.font",
    "ToolBar.foreground",
    "ToolBar.highlight",
    "ToolBar.isRollover",
    "ToolBar.light",
    "ToolBar.nonrolloverBorder",
    "ToolBar.rolloverBorder",
    "ToolBar.separatorSize",
    "ToolBar.shadow",
    "ToolBarButton.insets",
    "ToolBarButton.margin",
    "ToolBarSeparatorUI",
    "ToolBarUI",
    "ToolTip.background",
    "ToolTip.backgroundInactive",
    "ToolTip.border",
    "ToolTip.borderInactive",
    "ToolTip.infoForeground",
    "ToolTip.font",
    "ToolTip.foreground",
    "ToolTip.foregroundInactive",
    "ToolTip.hideAccelerator",
    "ToolTipManager.enableToolTipMode",
    "ToolTipUI",
    "ToolWindow.Header.background",
    "ToolWindow.Header.borderColor",
    "ToolWindow.Header.inactiveBackground",
    "ToolWindow.HeaderCloseButton.background",
    "ToolWindow.HeaderTab.selectedBackground",
    "ToolWindow.HeaderTab.selectedInactiveBackground",
    "ToolWindow.header.font.size",
    "ToolWindow.overridden.header.font.size.offset",
    "Tree.actionMap",
    "Tree.ancestorInputMap",
    "Tree.background",
    "Tree.changeSelectionWithFocus",
    "Tree.closedIcon",
    "Tree.collapsedIcon",
    "Tree.drawsFocusBorderAroundIcon",
    "Tree.dropCellBackground",
    "Tree.dropLineColor",
    "Tree.editorBorder",
    "Tree.expandedIcon",
    "Tree.focusInputMap",
    "Tree.focusInputMap.RightToLeft",
    "Tree.font",
    "Tree.foreground",
    "Tree.hash",
    "Tree.leafIcon",
    "Tree.leftChildIndent",
    "Tree.line",
    "Tree.lineTypeDashed",
    "Tree.openIcon",
    "Tree.paintLines",
    "Tree.repaintWholeRow",
    "Tree.rightChildIndent",
    "Tree.rightToLeftCollapsedIcon",
    "Tree.rowHeight",
    "Tree.scrollsOnExpand",
    "Tree.selectionBackground",
    "Tree.selectionBorderColor",
    "Tree.selectionForeground",
    "Tree.selectionInactiveBackground",
    "Tree.selectionInactiveForeground",
    "Tree.textBackground",
    "Tree.textForeground",
    "Tree.timeFactor",
    "TreeUI",
    "ValidationTooltip.errorBackground",
    "ValidationTooltip.errorBorderColor",
    "ValidationTooltip.maxWidth",
    "ValidationTooltip.warningBackground",
    "ValidationTooltip.warningBorderColor",
    "Viewport.background",
    "Viewport.font",
    "Viewport.foreground",
    "ViewportUI",
    "_SecurityDecisionIcon",
    "activeCaption",
    "activeCaptionBorder",
    "activeCaptionText",
    "com.apple.laf.AquaButtonCheckBoxUI",
    "com.apple.laf.AquaButtonRadioUI",
    "com.apple.laf.AquaButtonToggleUI",
    "com.apple.laf.AquaButtonUI",
    "com.apple.laf.AquaComboBoxUI",
    "com.apple.laf.AquaEditorPaneUI",
    "com.apple.laf.AquaFileChooserUI",
    "com.apple.laf.AquaInternalFrameUI",
    "com.apple.laf.AquaLabelUI",
    "com.apple.laf.AquaListUI",
    "com.apple.laf.AquaMenuBarUI",
    "com.apple.laf.AquaMenuItemUI",
    "com.apple.laf.AquaMenuUI",
    "com.apple.laf.AquaPanelUI",
    "com.apple.laf.AquaPopupMenuSeparatorUI",
    "com.apple.laf.AquaPopupMenuUI",
    "com.apple.laf.AquaProgressBarUI",
    "com.apple.laf.AquaRootPaneUI",
    "com.apple.laf.AquaScrollBarUI",
    "com.apple.laf.AquaScrollPaneUI",
    "com.apple.laf.AquaSliderUI",
    "com.apple.laf.AquaSpinnerUI",
    "com.apple.laf.AquaSplitPaneDividerUI",
    "com.apple.laf.AquaSplitPaneUI",
    "com.apple.laf.AquaTabbedPaneContrastUI",
    "com.apple.laf.AquaTabbedPaneCopyFromBasicUI",
    "com.apple.laf.AquaTabbedPaneUI",
    "com.apple.laf.AquaTableHeaderUI",
    "com.apple.laf.AquaTableUI",
    "com.apple.laf.AquaTextFieldUI",
    "com.apple.laf.AquaTextPaneUI",
    "com.apple.laf.AquaTextPasswordFieldUI",
    "com.apple.laf.AquaToolBarSeparatorUI",
    "com.apple.laf.AquaToolBarUI",
    "com.apple.laf.AquaToolTipUI",
    "com.apple.laf.AquaTreeUI",
    "com.intellij.ide.ui.laf.darcula.DarculaTableHeaderUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaButtonUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaCheckBoxUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaComboBoxUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaEditorPaneUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaLabelUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaPasswordFieldUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaProgressBarUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaRadioButtonUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaRootPaneUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaSeparatorUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaSpinnerUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaTabbedPaneUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaTextAreaUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaTextFieldUI",
    "com.intellij.ide.ui.laf.darcula.ui.DarculaTreeUI",
    "com.intellij.ide.ui.laf.intellij.IntelliJCheckBoxUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJButtonUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJCheckBoxUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJComboBoxUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJPasswordFieldUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJProgressBarUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJRadioButtonUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJSpinnerUI",
    "com.intellij.ide.ui.laf.intellij.MacIntelliJTextFieldUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJButtonUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJCheckBoxUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJComboBoxUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJPasswordFieldUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJRadioButtonUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJSliderUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJSpinnerUI",
    "com.intellij.ide.ui.laf.intellij.WinIntelliJTextFieldUI",
    "com.intellij.ui.components.JBScrollBar",
    "com.intellij.ui.mac.MacPopupMenuUI",
    "com.intellij.ui.plaf.beg.BegTableUI",
    "com.sun.java.swing.plaf.windows.WindowsLabelUI",
    "com.sun.java.swing.plaf.windows.WindowsRootPaneUI",
    "control",
    "controlDkShadow",
    "controlHighlight",
    "controlLtHighlight",
    "controlShadow",
    "controlText",
    "darcula.background",
    "darcula.caretForeground",
    "darcula.foreground",
    "darcula.inactiveBackground",
    "darcula.selectionBackground",
    "darcula.selectionBackgroundInactive",
    "darcula.selectionForeground",
    "darcula.selectionForegroundInactive",
    "darcula.selectionInactiveBackground",
    "darcula.selectionInactiveForeground",
    "darcula.textBackground",
    "darcula.textForeground",
    "desktop",
    "grayFilter",
    "hidpi.scaleFactor",
    "html.missingImage",
    "html.pendingImage",
    "inactiveCaption",
    "inactiveCaptionBorder",
    "inactiveCaptionText",
    "info",
    "infoText",
    "intellijlaf.background",
    "intellijlaf.caretForeground",
    "intellijlaf.foreground",
    "intellijlaf.inactiveBackground",
    "intellijlaf.inactiveForeground",
    "intellijlaf.selectionBackground",
    "intellijlaf.selectionBackgroundInactive",
    "intellijlaf.selectionForeground",
    "intellijlaf.selectionForegroundInactive",
    "intellijlaf.selectionInactiveBackground",
    "intellijlaf.textBackground",
    "intellijlaf.textForeground",
    "intellijlaf_native.acceleratorSelectionForeground",
    "intellijlaf_native.background",
    "intellijlaf_native.caretForeground",
    "intellijlaf_native.disabledForeground",
    "intellijlaf_native.foreground",
    "intellijlaf_native.inactiveBackground",
    "intellijlaf_native.inactiveForeground",
    "intellijlaf_native.selectionBackground",
    "intellijlaf_native.selectionBackgroundInactive",
    "intellijlaf_native.selectionForeground",
    "intellijlaf_native.selectionForegroundInactive",
    "intellijlaf_native.selectionInactiveBackground",
    "intellijlaf_native.textBackground",
    "intellijlaf_native.textForeground",
    "javax.swing.JLabel.userStyleSheet",
    "javax.swing.plaf.basic.BasicButtonUI",
    "javax.swing.plaf.basic.BasicCheckBoxUI",
    "javax.swing.plaf.basic.BasicColorChooserUI",
    "javax.swing.plaf.basic.BasicComboBoxUI",
    "javax.swing.plaf.basic.BasicDesktopIconUI",
    "javax.swing.plaf.basic.BasicEditorPaneUI",
    "javax.swing.plaf.basic.BasicFileChooserUI",
    "javax.swing.plaf.basic.BasicFormattedTextFieldUI",
    "javax.swing.plaf.basic.BasicInternalFrameUI",
    "javax.swing.plaf.basic.BasicLabelUI",
    "javax.swing.plaf.basic.BasicListUI",
    "javax.swing.plaf.basic.BasicMenuBarUI",
    "javax.swing.plaf.basic.BasicMenuItemUI",
    "javax.swing.plaf.basic.BasicMenuUI",
    "javax.swing.plaf.basic.BasicOptionalPaneUI",
    "javax.swing.plaf.basic.BasicPanelUI",
    "javax.swing.plaf.basic.BasicPasswordFieldUI",
    "javax.swing.plaf.basic.BasicPopupMenuSeparatorUI",
    "javax.swing.plaf.basic.BasicPopupMenuUI",
    "javax.swing.plaf.basic.BasicProgressBarUI",
    "javax.swing.plaf.basic.BasicRadioButtonMenuItemUI",
    "javax.swing.plaf.basic.BasicRadioButtonUI",
    "javax.swing.plaf.basic.BasicRootPaneUI",
    "javax.swing.plaf.basic.BasicScrollBarUI",
    "javax.swing.plaf.basic.BasicScrollPaneUI",
    "javax.swing.plaf.basic.BasicSeparatorUI",
    "javax.swing.plaf.basic.BasicSliderUI",
    "javax.swing.plaf.basic.BasicSpinnerUI",
    "javax.swing.plaf.basic.BasicSplitPaneUI",
    "javax.swing.plaf.basic.BasicTabbedPaneUI",
    "javax.swing.plaf.basic.BasicTableHeaderUI",
    "javax.swing.plaf.basic.BasicTableUI",
    "javax.swing.plaf.basic.BasicTextAreaUI",
    "javax.swing.plaf.basic.BasicTextFieldUI",
    "javax.swing.plaf.basic.BasicTextPaneUI",
    "javax.swing.plaf.basic.BasicToolBarSeparatorUI",
    "javax.swing.plaf.basic.BasicToolBarUI",
    "javax.swing.plaf.basic.BasicToolTipUI",
    "javax.swing.plaf.basic.BasicTreeUI",
    "javax.swing.plaf.basic.BasicViewportUI",
    "javax.swing.plaf.metal.MetalButtonUI",
    "javax.swing.plaf.metal.MetalCheckBoxUI",
    "javax.swing.plaf.metal.MetalComboBoxUI",
    "javax.swing.plaf.metal.MetalDesktopIconUI",
    "javax.swing.plaf.metal.MetalFileChooserUI",
    "javax.swing.plaf.metal.MetalInternalFrameUI",
    "javax.swing.plaf.metal.MetalLabelUI",
    "javax.swing.plaf.metal.MetalMenuBarUI",
    "javax.swing.plaf.metal.MetalProgressBarUI",
    "javax.swing.plaf.metal.MetalRadioButtonUI",
    "javax.swing.plaf.metal.MetalRootPaneUI",
    "javax.swing.plaf.metal.MetalScrollBarUI",
    "javax.swing.plaf.metal.MetalScrollPaneUI",
    "javax.swing.plaf.metal.MetalSeparatorUI",
    "javax.swing.plaf.metal.MetalSliderUI",
    "javax.swing.plaf.metal.MetalSplitPaneUI",
    "javax.swing.plaf.metal.MetalTabbedPaneUI",
    "javax.swing.plaf.metal.MetalTextFieldUI",
    "javax.swing.plaf.metal.MetalToolBarUI",
    "javax.swing.plaf.metal.MetalToolTipUI",
    "javax.swing.plaf.metal.MetalTreeUI",
    "javax.swing.plaf.synth.SynthLookAndFeel",
    "link.foreground",
    "link.hover.foreground",
    "link.pressed.foreground",
    "link.visited.foreground",
    "menu",
    "menuText",
    "scrollbar",
    "text",
    "text.grayFilter",
    "textHighlight",
    "textHighlightText",
    "textInactiveText",
    "textText",
    "window",
    "windowBorder",
    "windowText"
  );


  public static final Set<String> NAMED_COLORS = ContainerUtil.immutableSet(
    "ActionButton.hoverBackground",
    "ActionButton.hoverBorderColor",
    "ActionButton.pressedBackground",
    "ActionButton.pressedBorderColor",
    "Borders.ContrastBorderColor",
    "Borders.color",
    "Button.default.shadowColor",
    "Button.default.endBackground",
    "Button.default.endBorderColor",
    "Button.default.focusColor",
    "Button.default.focusedBorderColor",
    "Button.default.startBackground",
    "Button.default.startBorderColor",
    "Button.disabledBorderColor",
    "Button.endBackground",
    "Button.endBorderColor",
    "Button.focusedBorderColor",
    "Button.shadowColor",
    "Button.startBackground",
    "Button.startBorderColor",
    "ComboBox.ArrowButton.background",
    "ComboBox.ArrowButton.disabledIconColor",
    "ComboBox.ArrowButton.iconColor",
    "ComboBox.ArrowButton.nonEditableBackground",
    "ComboBox.disabledForeground",
    "ComboBox.nonEditableBackground",
    "ComboBoxButton.background",
    "ComboPopup.border",
    "CompletionPopup.background",
    "CompletionPopup.foreground",
    "CompletionPopup.infoForeground",
    "CompletionPopup.matchForeground",
    "CompletionPopup.matchSelectionForeground",
    "CompletionPopup.nonFocusedState",
    "CompletionPopup.selectionBackground",
    "CompletionPopup.selectionForeground",
    "CompletionPopup.selectionInfoForeground",
    "CompletionPopup.selectionInactiveBackground",
    "Component.borderColor",
    "Component.disabledBorderColor",
    "Component.errorFocusColor",
    "Component.focusColor",
    "Component.focusedBorderColor",
    "Component.inactiveErrorFocusColor",
    "Component.inactiveWarningFocusColor",
    "Component.warningFocusColor",
    "Counter.background",
    "Counter.foreground",
    "DragAndDrop.areaBackground",
    "DragAndDrop.areaBorderColor",
    "DragAndDrop.areaForeground",
    "Editor.background",
    "Editor.foreground",
    "Editor.shortcutForeground",
    "EditorTabs.borderColor",
    "EditorTabs.inactiveMaskColor",
    "EditorTabs.selectedBackground",
    "EditorTabs.selectedForeground",
    "EditorTabs.underlineColor",
    "FileColor.Blue",
    "FileColor.Green",
    "FileColor.Orange",
    "FileColor.Rose",
    "FileColor.Violet",
    "FileColor.Yellow",
    "FlameGraph.JVMBackground",
    "FlameGraph.JVMFocusBackground",
    "FlameGraph.JVMFocusSearchNotMatchedBackground",
    "FlameGraph.JVMSearchNotMatchedBackground",
    "FlameGraph.frameForeground",
    "FlameGraph.nativeBackground",
    "FlameGraph.nativeFocusBackground",
    "FlameGraph.nativeFocusSearchNotMatchedBackground",
    "FlameGraph.nativeSearchNotMatchedBackground",
    "FlameGraph.parentBackground",
    "FlameGraph.parentFocusBackground",
    "FlameGraph.parentFocusSearchNotMatchedBackground",
    "FlameGraph.parentSearchNotMatchedBackground",
    "Git.Log.Ref.Head",
    "Git.Log.Ref.LocalBranch",
    "Git.Log.Ref.Other",
    "Git.Log.Ref.RemoteBranch",
    "Git.Log.Ref.Tag",
    "Group.separatorColor",
    "Group.disabledSeparatorColor",
    "GutterTooltip.infoForeground",
    "GutterTooltip.lineSeparatorColor",
    "HeaderColor.inactive",
    "HelpTooltip.background",
    "HelpTooltip.borderColor",
    "HelpTooltip.foreground",
    "HelpTooltip.shortcutForeground",
    "Hg.Log.Ref.Bookmark",
    "Hg.Log.Ref.Branch",
    "Hg.Log.Ref.ClosedBranch",
    "Hg.Log.Ref.Head",
    "Hg.Log.Ref.LocalTag",
    "Hg.Log.Ref.MqTag",
    "Hg.Log.Ref.Tag",
    "Hg.Log.Ref.Tip",
    "InformationHint.borderColor",
    "Label.disabledForeground",
    "Label.disabledText",
    "Label.errorForeground",
    "Label.foreground",
    "Label.infoForeground",
    "Label.selectedDisabledForeground",
    "Label.selectedForeground",
    "Menu.borderColor",
    "MenuBar.borderColor",
    "NavBar.borderColor",
    "Notification.MoreButton.background",
    "Notification.MoreButton.foreground",
    "Notification.MoreButton.innerBorderColor",
    "Notification.ToolWindow.errorBackground",
    "Notification.ToolWindow.errorBorderColor",
    "Notification.ToolWindow.errorForeground",
    "Notification.ToolWindow.informativeBackground",
    "Notification.ToolWindow.informativeBorderColor",
    "Notification.ToolWindow.informativeForeground",
    "Notification.ToolWindow.warningBackground",
    "Notification.ToolWindow.warningBorderColor",
    "Notification.ToolWindow.warningForeground",
    "Notification.background",
    "Notification.borderColor",
    "Notification.errorBackground",
    "Notification.errorBorderColor",
    "Notification.errorForeground",
    "Notification.foreground",
    "Panel.background",
    "ParameterInfo.background",
    "ParameterInfo.borderColor",
    "ParameterInfo.currentOverloadBackground",
    "ParameterInfo.currentParameterForeground",
    "ParameterInfo.disabledForeground",
    "ParameterInfo.foreground",
    "ParameterInfo.infoForeground",
    "ParameterInfo.lineSeparatorColor",
    "Plugins.Button.installBackground",
    "Plugins.Button.installBorderColor",
    "Plugins.Button.installFillBackground",
    "Plugins.Button.installFillForeground",
    "Plugins.Button.installFocusedBackground",
    "Plugins.Button.installForeground",
    "Plugins.Button.updateBackground",
    "Plugins.Button.updateBorderColor",
    "Plugins.Button.updateForeground",
    "Plugins.SearchField.background",
    "Plugins.SearchField.borderColor",
    "Plugins.SectionHeader.background",
    "Plugins.SectionHeader.foreground",
    "Plugins.Tab.hoverBackground",
    "Plugins.Tab.selectedBackground",
    "Plugins.Tab.selectedForeground",
    "Plugins.background",
    "Plugins.disabledForeground",
    "Plugins.eapTagBackground",
    "Plugins.lightSelectionBackground",
    "Plugins.tagBackground",
    "Plugins.tagForeground",
    "Popup.Advertiser.background",
    "Popup.Advertiser.borderColor",
    "Popup.Advertiser.foreground",
    "Popup.Advertiser.borderInsets",
    "Popup.Header.activeBackground",
    "Popup.Header.inactiveBackground",
    "Popup.Toolbar.background",
    "Popup.Toolbar.borderColor",
    "Popup.borderColor",
    "Popup.paintBorder",
    "Popup.innerBorderColor",
    "Popup.inactiveBorderColor",
    "Popup.separatorColor",
    "Popup.separatorForeground",
    "ProgressBar.failedColor",
    "ProgressBar.failedEndColor",
    "ProgressBar.indeterminateEndColor",
    "ProgressBar.indeterminateStartColor",
    "ProgressBar.passedColor",
    "ProgressBar.passedEndColor",
    "ProgressBar.progressColor",
    "ProgressBar.trackColor",
    "SearchEverywhere.Advertiser.background",
    "SearchEverywhere.Advertiser.foreground",
    "SearchEverywhere.Advertiser.borderInsets",
    "SearchEverywhere.Header.background",
    "SearchEverywhere.List.separatorColor",
    "SearchEverywhere.List.separatorForeground",
    "SearchEverywhere.SearchField.background",
    "SearchEverywhere.SearchField.borderColor",
    "SearchEverywhere.SearchField.infoForeground",
    "SearchEverywhere.Tab.selectedBackground",
    "SearchEverywhere.Tab.selectedForeground",
    "SearchMatch.endBackground",
    "SearchMatch.startBackground",
    "Separator.separatorColor",
    "SidePanel.background",
    "StatusBar.borderColor",
    "TabbedPane.contentAreaColor",
    "TabbedPane.disabledForeground",
    "TabbedPane.disabledUnderlineColor",
    "TabbedPane.focusColor",
    "TabbedPane.hoverColor",
    "TabbedPane.underlineColor",
    "Table.lightSelectionBackground",
    "Table.lightSelectionForeground",
    "Table.lightSelectionInactiveBackground",
    "Table.lightSelectionInactiveForeground",
    "Table.stripeColor",
    "TitlePane.background",
    "ToolWindow.Button.hoverBackground",
    "ToolWindow.Button.selectedBackground",
    "ToolWindow.Button.selectedForeground",
    "ToolWindow.Header.background",
    "ToolWindow.Header.borderColor",
    "ToolWindow.Header.inactiveBackground",
    "ToolWindow.HeaderCloseButton.background",
    "ToolWindow.HeaderTab.hoverBackground",
    "ToolWindow.HeaderTab.hoverInactiveBackground",
    "ToolWindow.HeaderTab.selectedBackground",
    "ToolWindow.HeaderTab.selectedInactiveBackground",
    "ValidationTooltip.errorBackground",
    "ValidationTooltip.errorBorderColor",
    "ValidationTooltip.warningBackground",
    "ValidationTooltip.warningBorderColor",
    "VersionControl.FileHistory.Commit.selectedBranchBackground",
    "VersionControl.Log.Commit.currentBranchBackground",
    "VersionControl.Log.Commit.unmatchedForeground",
    "VersionControl.Ref.backgroundBase",
    "VersionControl.Ref.backgroundBrightness",
    "VersionControl.Ref.foreground",
    "WelcomeScreen.Projects.background",
    "WelcomeScreen.Projects.selectionBackground",
    "WelcomeScreen.Projects.selectionInactiveBackground",
    "WelcomeScreen.background",
    "WelcomeScreen.borderColor",
    "WelcomeScreen.captionBackground",
    "WelcomeScreen.captionForeground",
    "WelcomeScreen.footerBackground",
    "WelcomeScreen.footerForeground",
    "WelcomeScreen.groupIconBorderColor",
    "WelcomeScreen.headerBackground",
    "WelcomeScreen.headerForeground",
    "WelcomeScreen.separatorColor",
    "Window.border",
    "ToggleButton.borderColor",
    "ToggleButton.buttonColor",
    "ToggleButton.onBackground",
    "ToggleButton.onForeground",
    "ToggleButton.offBackground",
    "ToggleButton.offForeground",
    "HeaderColor.active",
    "ToolTip.Actions.background",
    "SpeedSearch.borderColor",
    "SpeedSearch.foreground",
    "SpeedSearch.background",
    "SpeedSearch.errorForeground",
    "DebuggerTabs.selectedBackground",
    "ToolTip.Actions.infoForeground"
  );

  public static final Set<String> ALL_KEYS = ContainerUtil.unmodifiableOrEmptySet(ContainerUtil.union(UI_DEFAULTS_KEYS, NAMED_COLORS));
}
